package edu.sc.seis.fissuresUtil.cache;

import edu.iris.Fissures.IfNetwork.NetworkAccess;
import edu.iris.Fissures.IfNetwork.NetworkFinder;
import edu.iris.Fissures.IfNetwork.NetworkId;
import edu.iris.Fissures.IfNetwork.NetworkNotFound;

public class NSNetworkFinder extends ProxyNetworkFinder {

    public NSNetworkFinder(ProxyNetworkDC netDC,
                           int numRetry,
                           RetryStrategy handler) {
        super();
        this.netDC = netDC;
        this.numRetry = numRetry;
        this.strategy = handler;
    }

    private synchronized ProxyNetworkFinder getFinder() {
        if(finder == null) {
            finder = new RetryNetworkFinder(new SynchronizedNetworkFinder(netDC.a_finder()),
                                            numRetry,
                                            strategy);
        }
        return finder;
    }

    public NetworkAccess retrieve_by_id(NetworkId id) throws NetworkNotFound {
        try {
            return getFinder().retrieve_by_id(id);
        } catch(Throwable e) {
            reset();
            return getFinder().retrieve_by_id(id);
        } // end of try-catch
    }

    public NetworkAccess[] retrieve_by_code(String code) throws NetworkNotFound {
        try {
            return getFinder().retrieve_by_code(code);
        } catch(Throwable e) {
            reset();
            return getFinder().retrieve_by_code(code);
        } // end of try-catch
    }

    public NetworkAccess[] retrieve_by_name(String name) throws NetworkNotFound {
        try {
            return getFinder().retrieve_by_name(name);
        } catch(Throwable e) {
            reset();
            return getFinder().retrieve_by_name(name);
        } // end of try-catch
    }

    public NetworkAccess[] retrieve_all() {
        try {
            return getFinder().retrieve_all();
        } catch(Throwable e) {
            reset();
            return getFinder().retrieve_all();
        } // end of try-catch
    }

    /**
     * the networkfinder gets revested in a retry here. this stays in line with
     * what happens in BulletProofVestFactory.vestNetworkFinder. just be aware
     * that any changes there need to be checked here, as well.
     */
    public void reset() {
        netDC.reset();
        if(finder != null) {
            finder._release();
        }
        finder = null;
    }

    public boolean hasCorbaObject() {
        return getCorbaObject() != null;
    }

    public NetworkFinder getCorbaObject() {
        return getFinder().getCorbaObject();
    }

    public String getServerDNS() {
        return netDC.getServerDNS();
    }

    public String getServerName() {
        return netDC.getServerName();
    }

    ProxyNetworkDC netDC;

    private ProxyNetworkFinder finder;

    private int numRetry;

    private RetryStrategy strategy;
}
